package com.browserproxy;

import android.content.Context;
import android.util.Log;

/**
 * Test class to verify the functionality of the Android Browser Proxy App
 */
public class AppFunctionalityTest {
    private static final String TAG = "AppFunctionalityTest";
    
    private final Context context;
    private final ProxyConfigurationService proxyConfigService;
    
    /**
     * Constructor for AppFunctionalityTest
     * 
     * @param context Application context
     */
    public AppFunctionalityTest(Context context) {
        this.context = context;
        this.proxyConfigService = new ProxyConfigurationService(context);
    }
    
    /**
     * Runs a basic test to verify that multiple browser instances can be opened
     * 
     * @param url URL to open in the browsers
     * @param browserPackage Package name of the browser to use
     * @param instanceCount Number of instances to launch
     * @return Test result with success status and details
     */
    public TestResult testMultipleBrowserInstances(String url, String browserPackage, int instanceCount) {
        Log.i(TAG, "Starting test: Multiple Browser Instances");
        Log.d(TAG, "Parameters: url=" + url + ", browserPackage=" + browserPackage + ", instanceCount=" + instanceCount);
        
        // Start the service
        boolean serviceStarted = proxyConfigService.start();
        if (!serviceStarted) {
            Log.e(TAG, "Failed to start proxy configuration service");
            return new TestResult(false, "Failed to start proxy configuration service");
        }
        
        try {
            // Launch browsers with free proxies
            int launchedCount = proxyConfigService.launchBrowsersWithFreeProxies(url, browserPackage, instanceCount);
            
            // Verify the number of launched instances
            boolean success = launchedCount > 0;
            String details = "Launched " + launchedCount + " out of " + instanceCount + " requested instances";
            
            if (success) {
                Log.i(TAG, "Test passed: " + details);
            } else {
                Log.e(TAG, "Test failed: " + details);
            }
            
            return new TestResult(success, details);
        } catch (Exception e) {
            Log.e(TAG, "Test failed with exception: " + e.getMessage());
            return new TestResult(false, "Exception: " + e.getMessage());
        } finally {
            // Stop the service
            proxyConfigService.stop();
        }
    }
    
    /**
     * Tests proxy rotation functionality
     * 
     * @param url URL to open in the browsers
     * @param browserPackage Package name of the browser to use
     * @param instanceCount Number of instances to launch
     * @return Test result with success status and details
     */
    public TestResult testProxyRotation(String url, String browserPackage, int instanceCount) {
        Log.i(TAG, "Starting test: Proxy Rotation");
        Log.d(TAG, "Parameters: url=" + url + ", browserPackage=" + browserPackage + ", instanceCount=" + instanceCount);
        
        // Start the service
        boolean serviceStarted = proxyConfigService.start();
        if (!serviceStarted) {
            Log.e(TAG, "Failed to start proxy configuration service");
            return new TestResult(false, "Failed to start proxy configuration service");
        }
        
        try {
            // Launch browsers with free proxies
            int launchedCount = proxyConfigService.launchBrowsersWithFreeProxies(url, browserPackage, instanceCount);
            
            if (launchedCount == 0) {
                Log.e(TAG, "Failed to launch any browser instances");
                return new TestResult(false, "Failed to launch any browser instances");
            }
            
            // Wait a bit for browsers to initialize
            Thread.sleep(5000);
            
            // Rotate proxies
            int rotatedCount = proxyConfigService.rotateAllProxies();
            
            // Verify rotation
            boolean success = rotatedCount > 0;
            String details = "Rotated proxies for " + rotatedCount + " out of " + launchedCount + " instances";
            
            if (success) {
                Log.i(TAG, "Test passed: " + details);
            } else {
                Log.e(TAG, "Test failed: " + details);
            }
            
            return new TestResult(success, details);
        } catch (Exception e) {
            Log.e(TAG, "Test failed with exception: " + e.getMessage());
            return new TestResult(false, "Exception: " + e.getMessage());
        } finally {
            // Stop the service
            proxyConfigService.stop();
        }
    }
    
    /**
     * Tests multiple browser types with the same proxy
     * 
     * @param url URL to open in the browsers
     * @param browserCount Number of different browsers to launch
     * @return Test result with success status and details
     */
    public TestResult testMultipleBrowserTypes(String url, int browserCount) {
        Log.i(TAG, "Starting test: Multiple Browser Types");
        Log.d(TAG, "Parameters: url=" + url + ", browserCount=" + browserCount);
        
        // Start the service
        boolean serviceStarted = proxyConfigService.start();
        if (!serviceStarted) {
            Log.e(TAG, "Failed to start proxy configuration service");
            return new TestResult(false, "Failed to start proxy configuration service");
        }
        
        try {
            // Launch multiple browser types with the same proxy
            int launchedCount = proxyConfigService.launchMultipleBrowsersWithSameProxy(url, browserCount);
            
            // Verify the number of launched instances
            boolean success = launchedCount > 0;
            String details = "Launched " + launchedCount + " different browsers with the same proxy";
            
            if (success) {
                Log.i(TAG, "Test passed: " + details);
            } else {
                Log.e(TAG, "Test failed: " + details);
            }
            
            return new TestResult(success, details);
        } catch (Exception e) {
            Log.e(TAG, "Test failed with exception: " + e.getMessage());
            return new TestResult(false, "Exception: " + e.getMessage());
        } finally {
            // Stop the service
            proxyConfigService.stop();
        }
    }
    
    /**
     * Tests country-specific proxies
     * 
     * @param url URL to open in the browsers
     * @param browserPackage Package name of the browser to use
     * @param country Country to filter proxies by
     * @param instanceCount Number of instances to launch
     * @return Test result with success status and details
     */
    public TestResult testCountryProxies(String url, String browserPackage, String country, int instanceCount) {
        Log.i(TAG, "Starting test: Country Proxies");
        Log.d(TAG, "Parameters: url=" + url + ", browserPackage=" + browserPackage + ", country=" + country + ", instanceCount=" + instanceCount);
        
        // Start the service
        boolean serviceStarted = proxyConfigService.start();
        if (!serviceStarted) {
            Log.e(TAG, "Failed to start proxy configuration service");
            return new TestResult(false, "Failed to start proxy configuration service");
        }
        
        try {
            // Launch browsers with proxies from the specified country
            int launchedCount = proxyConfigService.launchBrowsersWithCountryProxies(url, browserPackage, country, instanceCount);
            
            // Verify the number of launched instances
            boolean success = launchedCount > 0;
            String details = "Launched " + launchedCount + " out of " + instanceCount + " requested instances with proxies from " + country;
            
            if (success) {
                Log.i(TAG, "Test passed: " + details);
            } else {
                Log.e(TAG, "Test failed: " + details);
            }
            
            return new TestResult(success, details);
        } catch (Exception e) {
            Log.e(TAG, "Test failed with exception: " + e.getMessage());
            return new TestResult(false, "Exception: " + e.getMessage());
        } finally {
            // Stop the service
            proxyConfigService.stop();
        }
    }
    
    /**
     * Runs all tests
     * 
     * @param url URL to open in the browsers
     * @return Summary of all test results
     */
    public String runAllTests(String url) {
        Log.i(TAG, "Running all tests with URL: " + url);
        
        StringBuilder summary = new StringBuilder();
        summary.append("Android Browser Proxy App - Test Results\n");
        summary.append("=======================================\n\n");
        
        // Get default browser
        String defaultBrowser = BrowserManager.CHROME_PACKAGE;
        BrowserManager browserManager = new BrowserManager(context);
        List<String> installedBrowsers = browserManager.getInstalledBrowsers();
        if (!installedBrowsers.isEmpty()) {
            defaultBrowser = installedBrowsers.get(0);
        }
        
        // Test 1: Multiple Browser Instances
        TestResult test1 = testMultipleBrowserInstances(url, defaultBrowser, 3);
        summary.append("1. Multiple Browser Instances: ").append(test1.isSuccess() ? "PASSED" : "FAILED").append("\n");
        summary.append("   Details: ").append(test1.getDetails()).append("\n\n");
        
        // Test 2: Proxy Rotation
        TestResult test2 = testProxyRotation(url, defaultBrowser, 2);
        summary.append("2. Proxy Rotation: ").append(test2.isSuccess() ? "PASSED" : "FAILED").append("\n");
        summary.append("   Details: ").append(test2.getDetails()).append("\n\n");
        
        // Test 3: Multiple Browser Types
        TestResult test3 = testMultipleBrowserTypes(url, 2);
        summary.append("3. Multiple Browser Types: ").append(test3.isSuccess() ? "PASSED" : "FAILED").append("\n");
        summary.append("   Details: ").append(test3.getDetails()).append("\n\n");
        
        // Test 4: Country Proxies
        TestResult test4 = testCountryProxies(url, defaultBrowser, "United States", 2);
        summary.append("4. Country Proxies: ").append(test4.isSuccess() ? "PASSED" : "FAILED").append("\n");
        summary.append("   Details: ").append(test4.getDetails()).append("\n\n");
        
        // Overall result
        boolean overallSuccess = test1.isSuccess() && test2.isSuccess() && test3.isSuccess() && test4.isSuccess();
        summary.append("Overall Result: ").append(overallSuccess ? "PASSED" : "FAILED").append("\n");
        
        Log.i(TAG, "All tests completed. Overall result: " + (overallSuccess ? "PASSED" : "FAILED"));
        
        return summary.toString();
    }
    
    /**
     * Class to represent a test result
     */
    public static class TestResult {
        private final boolean success;
        private final String details;
        
        /**
         * Constructor for TestResult
         * 
         * @param success Whether the test passed
         * @param details Details about the test result
         */
        public TestResult(boolean success, String details) {
            this.success = success;
            this.details = details;
        }
        
        /**
         * Gets whether the test passed
         * 
         * @return true if passed, false otherwise
         */
        public boolean isSuccess() {
            return success;
        }
        
        /**
         * Gets details about the test result
         * 
         * @return Test result details
         */
        public String getDetails() {
            return details;
        }
    }
}
