package com.browserproxy;

import android.content.Context;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * BrowserAutomationService handles the automation of multiple browser instances
 * with different proxy configurations.
 */
public class BrowserAutomationService {
    private static final String TAG = "BrowserAutomationService";
    
    private final Context context;
    private final BrowserManager browserManager;
    private final ScheduledExecutorService scheduler;
    private boolean isRunning = false;
    
    /**
     * Constructor for BrowserAutomationService
     * 
     * @param context Application context
     */
    public BrowserAutomationService(Context context) {
        this.context = context;
        this.browserManager = new BrowserManager(context);
        this.scheduler = Executors.newScheduledThreadPool(1);
    }
    
    /**
     * Starts the browser automation service
     * 
     * @return true if started successfully, false otherwise
     */
    public boolean start() {
        if (isRunning) {
            Log.w(TAG, "Service is already running");
            return false;
        }
        
        try {
            // Start status monitoring
            scheduler.scheduleAtFixedRate(
                    this::monitorBrowserInstances,
                    0,
                    5,
                    TimeUnit.SECONDS
            );
            
            isRunning = true;
            Log.i(TAG, "Browser automation service started");
            return true;
        } catch (Exception e) {
            Log.e(TAG, "Error starting service: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Stops the browser automation service
     * 
     * @return true if stopped successfully, false otherwise
     */
    public boolean stop() {
        if (!isRunning) {
            Log.w(TAG, "Service is not running");
            return false;
        }
        
        try {
            // Stop all scheduled tasks
            scheduler.shutdown();
            
            // Close all browser instances
            List<BrowserInstance> activeBrowsers = browserManager.getActiveBrowsers();
            for (BrowserInstance instance : activeBrowsers) {
                browserManager.closeBrowser(instance.getInstanceId());
            }
            
            isRunning = false;
            Log.i(TAG, "Browser automation service stopped");
            return true;
        } catch (Exception e) {
            Log.e(TAG, "Error stopping service: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Launches multiple browser instances with the same URL and different proxy configurations
     * 
     * @param url URL to open in the browsers
     * @param proxyConfigs List of proxy configurations
     * @param browserPackage Package name of the browser to launch
     * @return List of instance IDs of the launched browsers
     */
    public List<Integer> launchMultipleBrowsers(String url, List<ProxyConfig> proxyConfigs, String browserPackage) {
        List<Integer> instanceIds = new ArrayList<>();
        
        for (ProxyConfig proxyConfig : proxyConfigs) {
            int instanceId = browserManager.launchBrowser(url, proxyConfig, browserPackage);
            if (instanceId != -1) {
                instanceIds.add(instanceId);
                Log.i(TAG, "Launched browser instance " + instanceId + " with proxy " + proxyConfig.getProxyAddress());
            } else {
                Log.e(TAG, "Failed to launch browser with proxy " + proxyConfig.getProxyAddress());
            }
            
            // Add a small delay between launches to avoid overwhelming the system
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                Log.e(TAG, "Sleep interrupted: " + e.getMessage());
            }
        }
        
        return instanceIds;
    }
    
    /**
     * Launches multiple browser instances with the same URL and proxy configuration but different browsers
     * 
     * @param url URL to open in the browsers
     * @param proxyConfig Proxy configuration
     * @param browserPackages List of browser package names
     * @return List of instance IDs of the launched browsers
     */
    public List<Integer> launchMultipleBrowserTypes(String url, ProxyConfig proxyConfig, List<String> browserPackages) {
        List<Integer> instanceIds = new ArrayList<>();
        
        for (String browserPackage : browserPackages) {
            int instanceId = browserManager.launchBrowser(url, proxyConfig, browserPackage);
            if (instanceId != -1) {
                instanceIds.add(instanceId);
                Log.i(TAG, "Launched browser instance " + instanceId + " with browser " + browserPackage);
            } else {
                Log.e(TAG, "Failed to launch browser " + browserPackage);
            }
            
            // Add a small delay between launches to avoid overwhelming the system
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                Log.e(TAG, "Sleep interrupted: " + e.getMessage());
            }
        }
        
        return instanceIds;
    }
    
    /**
     * Gets a list of all active browser instances
     * 
     * @return List of active browser instances
     */
    public List<BrowserInstance> getActiveBrowserInstances() {
        return browserManager.getActiveBrowsers();
    }
    
    /**
     * Gets a list of installed browsers on the device
     * 
     * @return List of package names of installed browsers
     */
    public List<String> getInstalledBrowsers() {
        return browserManager.getInstalledBrowsers();
    }
    
    /**
     * Refreshes all active browser instances
     * 
     * @return Number of successfully refreshed instances
     */
    public int refreshAllBrowsers() {
        int successCount = 0;
        List<BrowserInstance> activeBrowsers = browserManager.getActiveBrowsers();
        
        for (BrowserInstance instance : activeBrowsers) {
            boolean success = browserManager.refreshBrowser(instance.getInstanceId());
            if (success) {
                successCount++;
            }
        }
        
        return successCount;
    }
    
    /**
     * Closes all active browser instances
     * 
     * @return Number of successfully closed instances
     */
    public int closeAllBrowsers() {
        int successCount = 0;
        List<BrowserInstance> activeBrowsers = browserManager.getActiveBrowsers();
        
        for (BrowserInstance instance : activeBrowsers) {
            boolean success = browserManager.closeBrowser(instance.getInstanceId());
            if (success) {
                successCount++;
            }
        }
        
        return successCount;
    }
    
    /**
     * Monitors browser instances and updates their status
     */
    private void monitorBrowserInstances() {
        List<BrowserInstance> activeBrowsers = browserManager.getActiveBrowsers();
        
        for (BrowserInstance instance : activeBrowsers) {
            // In a real implementation, we would check if the browser is still running
            // and update its status accordingly. For non-rooted devices, this is challenging
            // as we don't have direct access to process information.
            
            // For now, we'll just log the current status
            Log.d(TAG, "Instance " + instance.getInstanceId() + 
                  " (" + instance.getBrowserPackage() + ") status: " + 
                  instance.getStatus() + ", uptime: " + 
                  (instance.getUptime() / 1000) + " seconds");
        }
    }
    
    /**
     * Checks if the service is running
     * 
     * @return true if running, false otherwise
     */
    public boolean isRunning() {
        return isRunning;
    }
}
