package com.browserproxy;

/**
 * Represents a browser instance with its configuration and status
 */
public class BrowserInstance {
    private int instanceId;
    private String browserPackage;
    private String currentUrl;
    private ProxyConfig proxyConfig;
    private BrowserStatus status;
    private long launchTime;

    /**
     * Constructor for BrowserInstance
     *
     * @param instanceId    Unique identifier for the browser instance
     * @param browserPackage Package name of the browser
     * @param currentUrl    Current URL loaded in the browser
     * @param proxyConfig   Proxy configuration used by the browser
     * @param status        Current status of the browser instance
     * @param launchTime    Time when the browser was launched (milliseconds)
     */
    public BrowserInstance(int instanceId, String browserPackage, String currentUrl,
                          ProxyConfig proxyConfig, BrowserStatus status, long launchTime) {
        this.instanceId = instanceId;
        this.browserPackage = browserPackage;
        this.currentUrl = currentUrl;
        this.proxyConfig = proxyConfig;
        this.status = status;
        this.launchTime = launchTime;
    }

    /**
     * Gets the instance ID
     *
     * @return Instance ID
     */
    public int getInstanceId() {
        return instanceId;
    }

    /**
     * Sets the instance ID
     *
     * @param instanceId New instance ID
     */
    public void setInstanceId(int instanceId) {
        this.instanceId = instanceId;
    }

    /**
     * Gets the browser package name
     *
     * @return Browser package name
     */
    public String getBrowserPackage() {
        return browserPackage;
    }

    /**
     * Sets the browser package name
     *
     * @param browserPackage New browser package name
     */
    public void setBrowserPackage(String browserPackage) {
        this.browserPackage = browserPackage;
    }

    /**
     * Gets the current URL
     *
     * @return Current URL
     */
    public String getCurrentUrl() {
        return currentUrl;
    }

    /**
     * Sets the current URL
     *
     * @param currentUrl New current URL
     */
    public void setCurrentUrl(String currentUrl) {
        this.currentUrl = currentUrl;
    }

    /**
     * Gets the proxy configuration
     *
     * @return Proxy configuration
     */
    public ProxyConfig getProxyConfig() {
        return proxyConfig;
    }

    /**
     * Sets the proxy configuration
     *
     * @param proxyConfig New proxy configuration
     */
    public void setProxyConfig(ProxyConfig proxyConfig) {
        this.proxyConfig = proxyConfig;
    }

    /**
     * Gets the status
     *
     * @return Status
     */
    public BrowserStatus getStatus() {
        return status;
    }

    /**
     * Sets the status
     *
     * @param status New status
     */
    public void setStatus(BrowserStatus status) {
        this.status = status;
    }

    /**
     * Gets the launch time
     *
     * @return Launch time (milliseconds)
     */
    public long getLaunchTime() {
        return launchTime;
    }

    /**
     * Sets the launch time
     *
     * @param launchTime New launch time (milliseconds)
     */
    public void setLaunchTime(long launchTime) {
        this.launchTime = launchTime;
    }

    /**
     * Gets the uptime of the browser instance
     *
     * @return Uptime in milliseconds
     */
    public long getUptime() {
        return System.currentTimeMillis() - launchTime;
    }

    /**
     * Gets a human-readable representation of the browser instance
     *
     * @return String representation
     */
    @Override
    public String toString() {
        return "BrowserInstance{" +
                "instanceId=" + instanceId +
                ", browserPackage='" + browserPackage + '\'' +
                ", currentUrl='" + currentUrl + '\'' +
                ", proxyConfig=" + proxyConfig +
                ", status=" + status +
                ", launchTime=" + launchTime +
                '}';
    }
}
