package com.browserproxy;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * BrowserManager is responsible for launching and managing multiple browser instances
 * with different proxy configurations.
 */
public class BrowserManager {
    private static final String TAG = "BrowserManager";
    
    // Common browser package names
    public static final String CHROME_PACKAGE = "com.android.chrome";
    public static final String FIREFOX_PACKAGE = "org.mozilla.firefox";
    public static final String SAMSUNG_BROWSER_PACKAGE = "com.sec.android.app.sbrowser";
    public static final String OPERA_PACKAGE = "com.opera.browser";
    public static final String BRAVE_PACKAGE = "com.brave.browser";
    public static final String EDGE_PACKAGE = "com.microsoft.emmx";
    
    private final Context context;
    private final Map<Integer, BrowserInstance> activeBrowsers;
    private final AtomicInteger instanceCounter;
    
    /**
     * Constructor for BrowserManager
     * 
     * @param context Application context
     */
    public BrowserManager(Context context) {
        this.context = context;
        this.activeBrowsers = new HashMap<>();
        this.instanceCounter = new AtomicInteger(0);
    }
    
    /**
     * Launches a browser with the specified URL and proxy configuration
     * 
     * @param url URL to open in the browser
     * @param proxyConfig Proxy configuration to use
     * @param browserPackage Package name of the browser to launch
     * @return Instance ID of the launched browser, or -1 if launch failed
     */
    public int launchBrowser(String url, ProxyConfig proxyConfig, String browserPackage) {
        if (!isBrowserInstalled(browserPackage)) {
            Log.e(TAG, "Browser " + browserPackage + " is not installed");
            return -1;
        }
        
        // Apply proxy configuration
        boolean proxyApplied = applyProxyConfiguration(proxyConfig);
        if (!proxyApplied) {
            Log.e(TAG, "Failed to apply proxy configuration");
            return -1;
        }
        
        // Launch browser with intent
        try {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setData(Uri.parse(url));
            intent.setPackage(browserPackage);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            
            context.startActivity(intent);
            
            // Create and store browser instance
            int instanceId = instanceCounter.incrementAndGet();
            BrowserInstance instance = new BrowserInstance(
                    instanceId,
                    browserPackage,
                    url,
                    proxyConfig,
                    BrowserStatus.ACTIVE,
                    System.currentTimeMillis()
            );
            
            activeBrowsers.put(instanceId, instance);
            return instanceId;
            
        } catch (Exception e) {
            Log.e(TAG, "Error launching browser: " + e.getMessage());
            return -1;
        }
    }
    
    /**
     * Closes a browser instance
     * 
     * @param instanceId ID of the browser instance to close
     * @return true if successful, false otherwise
     */
    public boolean closeBrowser(int instanceId) {
        BrowserInstance instance = activeBrowsers.get(instanceId);
        if (instance == null) {
            Log.e(TAG, "Browser instance " + instanceId + " not found");
            return false;
        }
        
        try {
            // For non-rooted devices, we can't force close the app
            // We can only clear the proxy settings and remove from our tracking
            clearProxyConfiguration();
            
            // Update instance status
            instance.setStatus(BrowserStatus.CLOSED);
            activeBrowsers.remove(instanceId);
            
            return true;
        } catch (Exception e) {
            Log.e(TAG, "Error closing browser: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Refreshes a browser instance by relaunching it with the same configuration
     * 
     * @param instanceId ID of the browser instance to refresh
     * @return true if successful, false otherwise
     */
    public boolean refreshBrowser(int instanceId) {
        BrowserInstance instance = activeBrowsers.get(instanceId);
        if (instance == null) {
            Log.e(TAG, "Browser instance " + instanceId + " not found");
            return false;
        }
        
        try {
            // Close the current instance
            closeBrowser(instanceId);
            
            // Launch a new instance with the same configuration
            int newInstanceId = launchBrowser(
                    instance.getCurrentUrl(),
                    instance.getProxyConfig(),
                    instance.getBrowserPackage()
            );
            
            return newInstanceId != -1;
        } catch (Exception e) {
            Log.e(TAG, "Error refreshing browser: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Gets a list of all active browser instances
     * 
     * @return List of active browser instances
     */
    public List<BrowserInstance> getActiveBrowsers() {
        return new ArrayList<>(activeBrowsers.values());
    }
    
    /**
     * Gets the status of a browser instance
     * 
     * @param instanceId ID of the browser instance
     * @return Status of the browser instance, or null if not found
     */
    public BrowserStatus getBrowserStatus(int instanceId) {
        BrowserInstance instance = activeBrowsers.get(instanceId);
        if (instance == null) {
            return null;
        }
        
        return instance.getStatus();
    }
    
    /**
     * Updates the status of a browser instance
     * 
     * @param instanceId ID of the browser instance
     * @param status New status
     * @return true if successful, false otherwise
     */
    public boolean updateBrowserStatus(int instanceId, BrowserStatus status) {
        BrowserInstance instance = activeBrowsers.get(instanceId);
        if (instance == null) {
            return false;
        }
        
        instance.setStatus(status);
        return true;
    }
    
    /**
     * Checks if a browser is installed on the device
     * 
     * @param packageName Package name of the browser
     * @return true if installed, false otherwise
     */
    public boolean isBrowserInstalled(String packageName) {
        try {
            context.getPackageManager().getPackageInfo(packageName, 0);
            return true;
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
    }
    
    /**
     * Gets a list of installed browsers on the device
     * 
     * @return List of package names of installed browsers
     */
    public List<String> getInstalledBrowsers() {
        List<String> installedBrowsers = new ArrayList<>();
        
        String[] browsers = {
                CHROME_PACKAGE,
                FIREFOX_PACKAGE,
                SAMSUNG_BROWSER_PACKAGE,
                OPERA_PACKAGE,
                BRAVE_PACKAGE,
                EDGE_PACKAGE
        };
        
        for (String browser : browsers) {
            if (isBrowserInstalled(browser)) {
                installedBrowsers.add(browser);
            }
        }
        
        return installedBrowsers;
    }
    
    /**
     * Applies proxy configuration for browser launch
     * 
     * @param proxyConfig Proxy configuration to apply
     * @return true if successful, false otherwise
     */
    private boolean applyProxyConfiguration(ProxyConfig proxyConfig) {
        // For non-rooted devices, we can use system properties
        // Note: This requires API level 24+ (Android 7.0+)
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                System.setProperty("http.proxyHost", proxyConfig.getHost());
                System.setProperty("http.proxyPort", String.valueOf(proxyConfig.getPort()));
                System.setProperty("https.proxyHost", proxyConfig.getHost());
                System.setProperty("https.proxyPort", String.valueOf(proxyConfig.getPort()));
                
                if (proxyConfig.getUsername() != null && !proxyConfig.getUsername().isEmpty()) {
                    System.setProperty("http.proxyUser", proxyConfig.getUsername());
                    System.setProperty("http.proxyPassword", proxyConfig.getPassword());
                    System.setProperty("https.proxyUser", proxyConfig.getUsername());
                    System.setProperty("https.proxyPassword", proxyConfig.getPassword());
                }
                
                return true;
            } else {
                Log.e(TAG, "Proxy configuration requires Android 7.0+");
                return false;
            }
        } catch (Exception e) {
            Log.e(TAG, "Error applying proxy configuration: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Clears proxy configuration
     * 
     * @return true if successful, false otherwise
     */
    private boolean clearProxyConfiguration() {
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                System.clearProperty("http.proxyHost");
                System.clearProperty("http.proxyPort");
                System.clearProperty("https.proxyHost");
                System.clearProperty("https.proxyPort");
                System.clearProperty("http.proxyUser");
                System.clearProperty("http.proxyPassword");
                System.clearProperty("https.proxyUser");
                System.clearProperty("https.proxyPassword");
                
                return true;
            } else {
                Log.e(TAG, "Proxy configuration requires Android 7.0+");
                return false;
            }
        } catch (Exception e) {
            Log.e(TAG, "Error clearing proxy configuration: " + e.getMessage());
            return false;
        }
    }
}
