package com.browserproxy;

import android.content.Context;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * A simple implementation of a proxy provider for free proxies
 * This class demonstrates how to integrate with free proxy services
 */
public class FreeProxyProvider {
    private static final String TAG = "FreeProxyProvider";
    
    // Sample list of free proxy servers (in a real app, these would be fetched from an API)
    private static final String[][] SAMPLE_FREE_PROXIES = {
        // host, port, country
        {"185.162.231.106", "80", "United States"},
        {"103.152.112.162", "80", "India"},
        {"103.117.192.14", "80", "Indonesia"},
        {"91.107.247.138", "53281", "Germany"},
        {"45.167.124.193", "9992", "Brazil"},
        {"185.82.99.42", "9091", "Russia"},
        {"103.155.217.156", "41472", "Indonesia"},
        {"103.48.68.36", "83", "India"},
        {"190.61.88.147", "8080", "Colombia"},
        {"45.173.6.98", "999", "Mexico"}
    };
    
    private final Context context;
    private final Random random;
    
    /**
     * Constructor for FreeProxyProvider
     * 
     * @param context Application context
     */
    public FreeProxyProvider(Context context) {
        this.context = context;
        this.random = new Random();
    }
    
    /**
     * Gets a list of free proxies
     * 
     * @param count Number of proxies to retrieve
     * @return List of proxy configurations
     */
    public List<ProxyConfig> getFreeProxies(int count) {
        List<ProxyConfig> proxies = new ArrayList<>();
        
        // In a real implementation, this would fetch proxies from a free proxy API
        // For demonstration purposes, we'll use the sample list
        
        for (int i = 0; i < count; i++) {
            // Get a random proxy from the sample list
            int index = random.nextInt(SAMPLE_FREE_PROXIES.length);
            String[] proxyData = SAMPLE_FREE_PROXIES[index];
            
            String host = proxyData[0];
            int port = Integer.parseInt(proxyData[1]);
            String country = proxyData[2];
            
            // Create a proxy configuration
            ProxyConfig proxyConfig = new ProxyConfig(
                    host,
                    port,
                    null,  // No authentication for free proxies
                    null,
                    ProxyType.UNKNOWN,  // Type is unknown for free proxies
                    country,
                    null,  // City is unknown
                    0      // No rotation
            );
            
            proxies.add(proxyConfig);
            
            Log.d(TAG, "Added free proxy: " + host + ":" + port + " (" + country + ")");
        }
        
        return proxies;
    }
    
    /**
     * Gets a list of free proxies from a specific country
     * 
     * @param country Country to filter by
     * @param count Number of proxies to retrieve
     * @return List of proxy configurations
     */
    public List<ProxyConfig> getFreeProxiesByCountry(String country, int count) {
        List<ProxyConfig> proxies = new ArrayList<>();
        List<String[]> matchingProxies = new ArrayList<>();
        
        // Find proxies matching the country
        for (String[] proxyData : SAMPLE_FREE_PROXIES) {
            if (proxyData[2].equalsIgnoreCase(country)) {
                matchingProxies.add(proxyData);
            }
        }
        
        // If no matching proxies found, return empty list
        if (matchingProxies.isEmpty()) {
            Log.w(TAG, "No free proxies found for country: " + country);
            return proxies;
        }
        
        // Get random proxies from the matching list
        for (int i = 0; i < Math.min(count, matchingProxies.size()); i++) {
            int index = random.nextInt(matchingProxies.size());
            String[] proxyData = matchingProxies.get(index);
            matchingProxies.remove(index);  // Remove to avoid duplicates
            
            String host = proxyData[0];
            int port = Integer.parseInt(proxyData[1]);
            
            // Create a proxy configuration
            ProxyConfig proxyConfig = new ProxyConfig(
                    host,
                    port,
                    null,  // No authentication for free proxies
                    null,
                    ProxyType.UNKNOWN,  // Type is unknown for free proxies
                    country,
                    null,  // City is unknown
                    0      // No rotation
            );
            
            proxies.add(proxyConfig);
            
            Log.d(TAG, "Added free proxy from " + country + ": " + host + ":" + port);
        }
        
        return proxies;
    }
    
    /**
     * Tests a proxy configuration to verify it's working
     * 
     * @param proxyConfig Proxy configuration to test
     * @return true if the proxy is working, false otherwise
     */
    public boolean testProxy(ProxyConfig proxyConfig) {
        // In a real implementation, this would test the proxy by making a request
        // For demonstration purposes, we'll simulate a test with random success
        
        boolean success = random.nextBoolean();
        
        if (success) {
            Log.d(TAG, "Proxy test successful: " + proxyConfig.getProxyAddress());
        } else {
            Log.d(TAG, "Proxy test failed: " + proxyConfig.getProxyAddress());
        }
        
        return success;
    }
}
