package com.browserproxy;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Main activity for the Android Browser Proxy App
 */
public class MainActivity extends Activity {
    private static final String TAG = "MainActivity";
    
    private ProxyConfigurationService proxyConfigService;
    private EditText urlEditText;
    private Spinner browserSpinner;
    private EditText instanceCountEditText;
    private Button launchButton;
    private Button rotateButton;
    private Button stopButton;
    private TextView statusTextView;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        
        // Initialize the proxy configuration service
        proxyConfigService = new ProxyConfigurationService(this);
        
        // Initialize UI components
        urlEditText = findViewById(R.id.url_edit_text);
        browserSpinner = findViewById(R.id.browser_spinner);
        instanceCountEditText = findViewById(R.id.instance_count_edit_text);
        launchButton = findViewById(R.id.launch_button);
        rotateButton = findViewById(R.id.rotate_button);
        stopButton = findViewById(R.id.stop_button);
        statusTextView = findViewById(R.id.status_text_view);
        
        // Set default URL
        urlEditText.setText("https://example.com");
        
        // Set up button click listeners
        launchButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                launchBrowsers();
            }
        });
        
        rotateButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                rotateProxies();
            }
        });
        
        stopButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                stopService();
            }
        });
        
        // Populate browser spinner
        populateBrowserSpinner();
        
        // Update status
        updateStatus("Ready to launch browsers with proxies");
    }
    
    @Override
    protected void onDestroy() {
        super.onDestroy();
        
        // Stop the service when the activity is destroyed
        if (proxyConfigService != null) {
            proxyConfigService.stop();
        }
    }
    
    /**
     * Populates the browser spinner with installed browsers
     */
    private void populateBrowserSpinner() {
        // In a real implementation, this would populate the spinner with installed browsers
        // For this example, we'll assume the spinner is already populated in the layout
    }
    
    /**
     * Launches browsers with proxies
     */
    private void launchBrowsers() {
        String url = urlEditText.getText().toString();
        if (url.isEmpty()) {
            Toast.makeText(this, "Please enter a URL", Toast.LENGTH_SHORT).show();
            return;
        }
        
        String browserPackage = getBrowserPackageFromSpinner();
        if (browserPackage == null) {
            Toast.makeText(this, "Please select a browser", Toast.LENGTH_SHORT).show();
            return;
        }
        
        int instanceCount;
        try {
            instanceCount = Integer.parseInt(instanceCountEditText.getText().toString());
            if (instanceCount <= 0 || instanceCount > 10) {
                Toast.makeText(this, "Instance count must be between 1 and 10", Toast.LENGTH_SHORT).show();
                return;
            }
        } catch (NumberFormatException e) {
            Toast.makeText(this, "Please enter a valid instance count", Toast.LENGTH_SHORT).show();
            return;
        }
        
        // Start the service
        boolean serviceStarted = proxyConfigService.start();
        if (!serviceStarted) {
            updateStatus("Failed to start proxy configuration service");
            return;
        }
        
        // Launch browsers with free proxies
        updateStatus("Launching " + instanceCount + " browser instances...");
        
        // Run in a background thread to avoid blocking the UI
        new Thread(new Runnable() {
            @Override
            public void run() {
                final int launchedCount = proxyConfigService.launchBrowsersWithFreeProxies(url, browserPackage, instanceCount);
                
                // Update UI on the main thread
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (launchedCount > 0) {
                            updateStatus("Successfully launched " + launchedCount + " browser instances");
                        } else {
                            updateStatus("Failed to launch any browser instances");
                        }
                    }
                });
            }
        }).start();
    }
    
    /**
     * Rotates proxies for all active browser instances
     */
    private void rotateProxies() {
        if (!proxyConfigService.isRunning()) {
            Toast.makeText(this, "Service is not running", Toast.LENGTH_SHORT).show();
            return;
        }
        
        updateStatus("Rotating proxies...");
        
        // Run in a background thread to avoid blocking the UI
        new Thread(new Runnable() {
            @Override
            public void run() {
                final int rotatedCount = proxyConfigService.rotateAllProxies();
                
                // Update UI on the main thread
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (rotatedCount > 0) {
                            updateStatus("Successfully rotated proxies for " + rotatedCount + " browser instances");
                        } else {
                            updateStatus("Failed to rotate any proxies");
                        }
                    }
                });
            }
        }).start();
    }
    
    /**
     * Stops the proxy configuration service
     */
    private void stopService() {
        if (!proxyConfigService.isRunning()) {
            Toast.makeText(this, "Service is not running", Toast.LENGTH_SHORT).show();
            return;
        }
        
        updateStatus("Stopping service...");
        
        // Run in a background thread to avoid blocking the UI
        new Thread(new Runnable() {
            @Override
            public void run() {
                final boolean stopped = proxyConfigService.stop();
                
                // Update UI on the main thread
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (stopped) {
                            updateStatus("Service stopped");
                        } else {
                            updateStatus("Failed to stop service");
                        }
                    }
                });
            }
        }).start();
    }
    
    /**
     * Gets the browser package name from the spinner
     * 
     * @return Browser package name
     */
    private String getBrowserPackageFromSpinner() {
        // In a real implementation, this would get the selected browser package from the spinner
        // For this example, we'll return Chrome as the default
        return BrowserManager.CHROME_PACKAGE;
    }
    
    /**
     * Updates the status text view
     * 
     * @param status Status message
     */
    private void updateStatus(String status) {
        Log.d(TAG, "Status: " + status);
        statusTextView.setText(status);
    }
}
