package com.browserproxy;

/**
 * Represents a proxy configuration with connection details
 */
public class ProxyConfig {
    private String host;
    private int port;
    private String username;
    private String password;
    private ProxyType type;
    private String country;
    private String city;
    private long rotationInterval;

    /**
     * Constructor for ProxyConfig
     *
     * @param host Proxy host address
     * @param port Proxy port number
     * @param username Username for authentication (can be null)
     * @param password Password for authentication (can be null)
     * @param type Type of proxy (residential, datacenter, etc.)
     * @param country Country of the proxy (can be null)
     * @param city City of the proxy (can be null)
     * @param rotationInterval Time interval for proxy rotation in milliseconds (0 for no rotation)
     */
    public ProxyConfig(String host, int port, String username, String password,
                      ProxyType type, String country, String city, long rotationInterval) {
        this.host = host;
        this.port = port;
        this.username = username;
        this.password = password;
        this.type = type;
        this.country = country;
        this.city = city;
        this.rotationInterval = rotationInterval;
    }

    /**
     * Gets the proxy host
     *
     * @return Proxy host
     */
    public String getHost() {
        return host;
    }

    /**
     * Sets the proxy host
     *
     * @param host New proxy host
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * Gets the proxy port
     *
     * @return Proxy port
     */
    public int getPort() {
        return port;
    }

    /**
     * Sets the proxy port
     *
     * @param port New proxy port
     */
    public void setPort(int port) {
        this.port = port;
    }

    /**
     * Gets the username for authentication
     *
     * @return Username
     */
    public String getUsername() {
        return username;
    }

    /**
     * Sets the username for authentication
     *
     * @param username New username
     */
    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * Gets the password for authentication
     *
     * @return Password
     */
    public String getPassword() {
        return password;
    }

    /**
     * Sets the password for authentication
     *
     * @param password New password
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * Gets the proxy type
     *
     * @return Proxy type
     */
    public ProxyType getType() {
        return type;
    }

    /**
     * Sets the proxy type
     *
     * @param type New proxy type
     */
    public void setType(ProxyType type) {
        this.type = type;
    }

    /**
     * Gets the country of the proxy
     *
     * @return Country
     */
    public String getCountry() {
        return country;
    }

    /**
     * Sets the country of the proxy
     *
     * @param country New country
     */
    public void setCountry(String country) {
        this.country = country;
    }

    /**
     * Gets the city of the proxy
     *
     * @return City
     */
    public String getCity() {
        return city;
    }

    /**
     * Sets the city of the proxy
     *
     * @param city New city
     */
    public void setCity(String city) {
        this.city = city;
    }

    /**
     * Gets the rotation interval
     *
     * @return Rotation interval in milliseconds
     */
    public long getRotationInterval() {
        return rotationInterval;
    }

    /**
     * Sets the rotation interval
     *
     * @param rotationInterval New rotation interval in milliseconds
     */
    public void setRotationInterval(long rotationInterval) {
        this.rotationInterval = rotationInterval;
    }

    /**
     * Checks if the proxy requires authentication
     *
     * @return true if authentication is required, false otherwise
     */
    public boolean requiresAuthentication() {
        return username != null && !username.isEmpty() && 
               password != null && !password.isEmpty();
    }

    /**
     * Gets the proxy address in the format host:port
     *
     * @return Proxy address
     */
    public String getProxyAddress() {
        return host + ":" + port;
    }

    /**
     * Gets a human-readable representation of the proxy configuration
     *
     * @return String representation
     */
    @Override
    public String toString() {
        return "ProxyConfig{" +
                "host='" + host + '\'' +
                ", port=" + port +
                ", username='" + (username != null ? "****" : "null") + '\'' +
                ", password='" + (password != null ? "****" : "null") + '\'' +
                ", type=" + type +
                ", country='" + country + '\'' +
                ", city='" + city + '\'' +
                ", rotationInterval=" + rotationInterval +
                '}';
    }
}
