package com.browserproxy;

import android.content.Context;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;

/**
 * ProxyConfigurationService integrates the browser automation and proxy management
 * to provide a complete solution for running multiple browser instances with different proxies.
 */
public class ProxyConfigurationService {
    private static final String TAG = "ProxyConfigService";
    
    private final Context context;
    private final BrowserAutomationService browserService;
    private final ProxyManager proxyManager;
    
    /**
     * Constructor for ProxyConfigurationService
     * 
     * @param context Application context
     */
    public ProxyConfigurationService(Context context) {
        this.context = context;
        this.browserService = new BrowserAutomationService(context);
        this.proxyManager = new ProxyManager(context);
    }
    
    /**
     * Starts the service
     * 
     * @return true if started successfully, false otherwise
     */
    public boolean start() {
        boolean proxyStarted = proxyManager.start();
        boolean browserStarted = browserService.start();
        
        if (!proxyStarted || !browserStarted) {
            Log.e(TAG, "Failed to start service: proxyStarted=" + proxyStarted + ", browserStarted=" + browserStarted);
            stop();  // Clean up if either failed
            return false;
        }
        
        Log.i(TAG, "Proxy configuration service started");
        return true;
    }
    
    /**
     * Stops the service
     * 
     * @return true if stopped successfully, false otherwise
     */
    public boolean stop() {
        boolean proxyStopped = true;
        boolean browserStopped = true;
        
        if (proxyManager.isRunning()) {
            proxyStopped = proxyManager.stop();
        }
        
        if (browserService.isRunning()) {
            browserStopped = browserService.stop();
        }
        
        if (!proxyStopped || !browserStopped) {
            Log.e(TAG, "Error stopping service: proxyStopped=" + proxyStopped + ", browserStopped=" + browserStopped);
            return false;
        }
        
        Log.i(TAG, "Proxy configuration service stopped");
        return true;
    }
    
    /**
     * Launches multiple browser instances with free proxies
     * 
     * @param url URL to open in the browsers
     * @param browserPackage Package name of the browser to launch
     * @param count Number of instances to launch
     * @return Number of successfully launched instances
     */
    public int launchBrowsersWithFreeProxies(String url, String browserPackage, int count) {
        // Get free proxies
        Log.d(TAG, "Getting " + count + " free proxies");
        List<ProxyConfig> proxies = proxyManager.getFreeProxies(count);
        
        if (proxies.isEmpty()) {
            Log.e(TAG, "No free proxies available");
            return 0;
        }
        
        // Launch browsers with the proxies
        Log.d(TAG, "Launching " + proxies.size() + " browser instances with free proxies");
        List<Integer> instanceIds = browserService.launchMultipleBrowsers(url, proxies, browserPackage);
        
        // Associate proxies with instances
        for (int i = 0; i < instanceIds.size(); i++) {
            int instanceId = instanceIds.get(i);
            ProxyConfig proxy = proxies.get(i);
            proxyManager.associateProxyWithInstance(instanceId, proxy);
        }
        
        Log.i(TAG, "Successfully launched " + instanceIds.size() + " browser instances with free proxies");
        return instanceIds.size();
    }
    
    /**
     * Launches multiple browser instances with free proxies from a specific country
     * 
     * @param url URL to open in the browsers
     * @param browserPackage Package name of the browser to launch
     * @param country Country to filter proxies by
     * @param count Number of instances to launch
     * @return Number of successfully launched instances
     */
    public int launchBrowsersWithCountryProxies(String url, String browserPackage, String country, int count) {
        // Get free proxies from the specified country
        Log.d(TAG, "Getting " + count + " free proxies from " + country);
        List<ProxyConfig> proxies = proxyManager.getFreeProxiesByCountry(country, count);
        
        if (proxies.isEmpty()) {
            Log.e(TAG, "No free proxies available for country: " + country);
            return 0;
        }
        
        // Launch browsers with the proxies
        Log.d(TAG, "Launching " + proxies.size() + " browser instances with proxies from " + country);
        List<Integer> instanceIds = browserService.launchMultipleBrowsers(url, proxies, browserPackage);
        
        // Associate proxies with instances
        for (int i = 0; i < instanceIds.size(); i++) {
            int instanceId = instanceIds.get(i);
            ProxyConfig proxy = proxies.get(i);
            proxyManager.associateProxyWithInstance(instanceId, proxy);
        }
        
        Log.i(TAG, "Successfully launched " + instanceIds.size() + " browser instances with proxies from " + country);
        return instanceIds.size();
    }
    
    /**
     * Launches multiple browser types with the same proxy
     * 
     * @param url URL to open in the browsers
     * @param count Number of browser types to launch
     * @return Number of successfully launched instances
     */
    public int launchMultipleBrowsersWithSameProxy(String url, int count) {
        // Get a free proxy
        Log.d(TAG, "Getting a free proxy");
        ProxyConfig proxy = proxyManager.getFreeProxy();
        
        if (proxy == null) {
            Log.e(TAG, "No free proxy available");
            return 0;
        }
        
        // Get installed browsers
        List<String> installedBrowsers = browserService.getInstalledBrowsers();
        if (installedBrowsers.isEmpty()) {
            Log.e(TAG, "No browsers installed");
            return 0;
        }
        
        // Limit to requested count
        if (installedBrowsers.size() > count) {
            installedBrowsers = installedBrowsers.subList(0, count);
        }
        
        // Launch different browsers with the same proxy
        Log.d(TAG, "Launching " + installedBrowsers.size() + " different browsers with the same proxy");
        List<Integer> instanceIds = browserService.launchMultipleBrowserTypes(url, proxy, installedBrowsers);
        
        // Associate proxy with instances
        for (int instanceId : instanceIds) {
            proxyManager.associateProxyWithInstance(instanceId, proxy);
        }
        
        Log.i(TAG, "Successfully launched " + instanceIds.size() + " different browsers with the same proxy");
        return instanceIds.size();
    }
    
    /**
     * Rotates proxies for all active browser instances
     * 
     * @return Number of successfully rotated proxies
     */
    public int rotateAllProxies() {
        int successCount = 0;
        List<BrowserInstance> activeBrowsers = browserService.getActiveBrowserInstances();
        
        for (BrowserInstance instance : activeBrowsers) {
            int instanceId = instance.getInstanceId();
            ProxyConfig newProxy = proxyManager.rotateProxy(instanceId);
            
            if (newProxy != null) {
                // Refresh the browser to apply the new proxy
                boolean refreshed = browserService.refreshBrowser(instanceId);
                if (refreshed) {
                    successCount++;
                }
            }
        }
        
        Log.i(TAG, "Successfully rotated proxies for " + successCount + " browser instances");
        return successCount;
    }
    
    /**
     * Gets the BrowserAutomationService
     * 
     * @return BrowserAutomationService
     */
    public BrowserAutomationService getBrowserService() {
        return browserService;
    }
    
    /**
     * Gets the ProxyManager
     * 
     * @return ProxyManager
     */
    public ProxyManager getProxyManager() {
        return proxyManager;
    }
}
