package com.browserproxy;

import android.content.Context;
import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * ProxyManager is responsible for managing proxy configurations,
 * including rotation, testing, and monitoring.
 */
public class ProxyManager {
    private static final String TAG = "ProxyManager";
    
    private final Context context;
    private final FreeProxyProvider freeProxyProvider;
    private final Map<Integer, ProxyConfig> activeProxies;
    private final ScheduledExecutorService scheduler;
    private boolean isRunning = false;
    
    /**
     * Constructor for ProxyManager
     * 
     * @param context Application context
     */
    public ProxyManager(Context context) {
        this.context = context;
        this.freeProxyProvider = new FreeProxyProvider(context);
        this.activeProxies = new HashMap<>();
        this.scheduler = Executors.newScheduledThreadPool(1);
    }
    
    /**
     * Starts the proxy manager
     * 
     * @return true if started successfully, false otherwise
     */
    public boolean start() {
        if (isRunning) {
            Log.w(TAG, "Proxy manager is already running");
            return false;
        }
        
        try {
            // Start proxy monitoring
            scheduler.scheduleAtFixedRate(
                    this::monitorProxies,
                    0,
                    30,
                    TimeUnit.SECONDS
            );
            
            isRunning = true;
            Log.i(TAG, "Proxy manager started");
            return true;
        } catch (Exception e) {
            Log.e(TAG, "Error starting proxy manager: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Stops the proxy manager
     * 
     * @return true if stopped successfully, false otherwise
     */
    public boolean stop() {
        if (!isRunning) {
            Log.w(TAG, "Proxy manager is not running");
            return false;
        }
        
        try {
            // Stop all scheduled tasks
            scheduler.shutdown();
            
            // Clear active proxies
            activeProxies.clear();
            
            isRunning = false;
            Log.i(TAG, "Proxy manager stopped");
            return true;
        } catch (Exception e) {
            Log.e(TAG, "Error stopping proxy manager: " + e.getMessage());
            return false;
        }
    }
    
    /**
     * Gets a free proxy configuration
     * 
     * @return Proxy configuration
     */
    public ProxyConfig getFreeProxy() {
        List<ProxyConfig> proxies = freeProxyProvider.getFreeProxies(1);
        if (proxies.isEmpty()) {
            Log.e(TAG, "No free proxies available");
            return null;
        }
        
        ProxyConfig proxy = proxies.get(0);
        
        // Test the proxy
        boolean isWorking = testProxy(proxy);
        if (!isWorking) {
            Log.w(TAG, "Free proxy is not working, trying another one");
            return getFreeProxy();  // Recursively try another proxy
        }
        
        return proxy;
    }
    
    /**
     * Gets multiple free proxy configurations
     * 
     * @param count Number of proxies to retrieve
     * @return List of proxy configurations
     */
    public List<ProxyConfig> getFreeProxies(int count) {
        List<ProxyConfig> proxies = freeProxyProvider.getFreeProxies(count * 2);  // Get extra for testing
        List<ProxyConfig> workingProxies = new ArrayList<>();
        
        for (ProxyConfig proxy : proxies) {
            if (workingProxies.size() >= count) {
                break;  // We have enough working proxies
            }
            
            // Test the proxy
            boolean isWorking = testProxy(proxy);
            if (isWorking) {
                workingProxies.add(proxy);
            }
        }
        
        if (workingProxies.isEmpty()) {
            Log.e(TAG, "No working free proxies found");
        } else if (workingProxies.size() < count) {
            Log.w(TAG, "Only found " + workingProxies.size() + " working proxies out of " + count + " requested");
        }
        
        return workingProxies;
    }
    
    /**
     * Gets free proxy configurations from a specific country
     * 
     * @param country Country to filter by
     * @param count Number of proxies to retrieve
     * @return List of proxy configurations
     */
    public List<ProxyConfig> getFreeProxiesByCountry(String country, int count) {
        List<ProxyConfig> proxies = freeProxyProvider.getFreeProxiesByCountry(country, count * 2);  // Get extra for testing
        List<ProxyConfig> workingProxies = new ArrayList<>();
        
        for (ProxyConfig proxy : proxies) {
            if (workingProxies.size() >= count) {
                break;  // We have enough working proxies
            }
            
            // Test the proxy
            boolean isWorking = testProxy(proxy);
            if (isWorking) {
                workingProxies.add(proxy);
            }
        }
        
        if (workingProxies.isEmpty()) {
            Log.e(TAG, "No working free proxies found for country: " + country);
        } else if (workingProxies.size() < count) {
            Log.w(TAG, "Only found " + workingProxies.size() + " working proxies for country: " + country + " out of " + count + " requested");
        }
        
        return workingProxies;
    }
    
    /**
     * Associates a proxy configuration with a browser instance
     * 
     * @param instanceId Browser instance ID
     * @param proxyConfig Proxy configuration
     */
    public void associateProxyWithInstance(int instanceId, ProxyConfig proxyConfig) {
        activeProxies.put(instanceId, proxyConfig);
        Log.d(TAG, "Associated proxy " + proxyConfig.getProxyAddress() + " with instance " + instanceId);
    }
    
    /**
     * Removes a proxy configuration association
     * 
     * @param instanceId Browser instance ID
     */
    public void removeProxyAssociation(int instanceId) {
        if (activeProxies.containsKey(instanceId)) {
            ProxyConfig proxy = activeProxies.get(instanceId);
            activeProxies.remove(instanceId);
            Log.d(TAG, "Removed proxy association for instance " + instanceId + ": " + proxy.getProxyAddress());
        }
    }
    
    /**
     * Gets the proxy configuration for a browser instance
     * 
     * @param instanceId Browser instance ID
     * @return Proxy configuration, or null if not found
     */
    public ProxyConfig getProxyForInstance(int instanceId) {
        return activeProxies.get(instanceId);
    }
    
    /**
     * Rotates the proxy for a browser instance
     * 
     * @param instanceId Browser instance ID
     * @return New proxy configuration, or null if rotation failed
     */
    public ProxyConfig rotateProxy(int instanceId) {
        ProxyConfig oldProxy = activeProxies.get(instanceId);
        if (oldProxy == null) {
            Log.e(TAG, "No proxy found for instance " + instanceId);
            return null;
        }
        
        // Get a new proxy
        ProxyConfig newProxy = getFreeProxy();
        if (newProxy == null) {
            Log.e(TAG, "Failed to get new proxy for rotation");
            return null;
        }
        
        // Update the association
        activeProxies.put(instanceId, newProxy);
        Log.d(TAG, "Rotated proxy for instance " + instanceId + ": " + 
              oldProxy.getProxyAddress() + " -> " + newProxy.getProxyAddress());
        
        return newProxy;
    }
    
    /**
     * Tests a proxy configuration to verify it's working
     * 
     * @param proxyConfig Proxy configuration to test
     * @return true if the proxy is working, false otherwise
     */
    public boolean testProxy(ProxyConfig proxyConfig) {
        return freeProxyProvider.testProxy(proxyConfig);
    }
    
    /**
     * Monitors active proxies and rotates them if needed
     */
    private void monitorProxies() {
        for (Map.Entry<Integer, ProxyConfig> entry : activeProxies.entrySet()) {
            int instanceId = entry.getKey();
            ProxyConfig proxy = entry.getValue();
            
            // Check if rotation is needed
            if (proxy.getRotationInterval() > 0) {
                long uptime = System.currentTimeMillis() - proxy.getLaunchTime();
                if (uptime >= proxy.getRotationInterval()) {
                    Log.d(TAG, "Rotation interval reached for proxy " + proxy.getProxyAddress() + 
                          " (instance " + instanceId + ")");
                    rotateProxy(instanceId);
                }
            }
            
            // Test the proxy
            boolean isWorking = testProxy(proxy);
            if (!isWorking) {
                Log.w(TAG, "Proxy " + proxy.getProxyAddress() + " is not working, rotating (instance " + 
                      instanceId + ")");
                rotateProxy(instanceId);
            }
        }
    }
    
    /**
     * Checks if the proxy manager is running
     * 
     * @return true if running, false otherwise
     */
    public boolean isRunning() {
        return isRunning;
    }
}
