package com.browserproxy;

import android.content.Context;
import android.util.Log;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * Test runner to execute all tests and generate a test report
 */
public class TestRunner {
    private static final String TAG = "TestRunner";
    
    private final Context context;
    private final AppFunctionalityTest testSuite;
    
    /**
     * Constructor for TestRunner
     * 
     * @param context Application context
     */
    public TestRunner(Context context) {
        this.context = context;
        this.testSuite = new AppFunctionalityTest(context);
    }
    
    /**
     * Runs all tests and generates a test report
     * 
     * @param url URL to use for testing
     * @return Path to the generated test report file
     */
    public String runTests(String url) {
        Log.i(TAG, "Running all tests with URL: " + url);
        
        // Run all tests
        String testResults = testSuite.runAllTests(url);
        
        // Generate test report
        String reportPath = generateTestReport(testResults);
        
        return reportPath;
    }
    
    /**
     * Generates a test report file
     * 
     * @param testResults Test results to include in the report
     * @return Path to the generated test report file
     */
    private String generateTestReport(String testResults) {
        try {
            // Create report directory if it doesn't exist
            File reportDir = new File(context.getExternalFilesDir(null), "reports");
            if (!reportDir.exists()) {
                reportDir.mkdirs();
            }
            
            // Create report file
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss", Locale.US);
            String timestamp = dateFormat.format(new Date());
            File reportFile = new File(reportDir, "test_report_" + timestamp + ".txt");
            
            // Write test results to file
            BufferedWriter writer = new BufferedWriter(new FileWriter(reportFile));
            writer.write("Android Browser Proxy App - Test Report\n");
            writer.write("Generated: " + new Date().toString() + "\n");
            writer.write("=======================================\n\n");
            writer.write(testResults);
            writer.close();
            
            Log.i(TAG, "Test report generated: " + reportFile.getAbsolutePath());
            
            return reportFile.getAbsolutePath();
        } catch (IOException e) {
            Log.e(TAG, "Error generating test report: " + e.getMessage());
            return null;
        }
    }
}
